#import "NSObject+Invocations.h"

@interface EGPerformOnMainThreadTrampoline : NSProxy {
    id object;
    BOOL waitUntilDone;
}

-(id)initWithObject:(id)inobject waitUntilDone:(BOOL)wait;
@end

@implementation EGPerformOnMainThreadTrampoline

-(id)initWithObject:(id)inobject waitUntilDone:(BOOL)wait
{
    object = [inobject retain];
    waitUntilDone = wait;
    
    return self;
}

-(void)dealloc
{
    [object release];
    [super dealloc];
}

- (void)forwardInvocation:(NSInvocation *)anInvocation 
{
    [anInvocation retainArguments];
    [object performSelectorOnMainThread:@selector(handleInvocation:)
                             withObject:anInvocation 
                          waitUntilDone:waitUntilDone];
}

-(NSMethodSignature*)methodSignatureForSelector:(SEL)aSelector
{
    return [object methodSignatureForSelector:aSelector];
}

@end

@implementation NSObject (PerformOnMainThreadTrampoline)

-(NSProxy*)performOnMainThreadWaitUntilDone:(BOOL)wait
{
    return [[[EGPerformOnMainThreadTrampoline alloc] initWithObject:self waitUntilDone:wait] autorelease];
}

-(void)handleInvocation:(NSInvocation *)anInvocation
{
    [anInvocation invokeWithTarget:self];
}

@end