//
//  DetailView.m
//  MobileDaxhotel
//
//  Created by Raphael Schweikert on 24.12.08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "DetailViewController.h"
#import "NSString+AdvancedSubstring.h"
#import "NSString+EntityDecoding.h"
#import "NSData+Base64Extensions.h"
#import "MobileDaxhotelAppDelegate.h"
#import "EpisodeImageView.h"

@implementation DetailViewController

- (void)displayEpisode:(NSDictionary *)episode ofSeries:(NSDictionary *)series updateTintColor:(BOOL)updateTintColor {
    UIColor *tintColor = [EpisodeSelectionTableController seriesColorForSeriesWithIdentifier:[series valueForKey:@"short_name"] asWeb:NO];
    if(updateTintColor) {
        self.navigationController.navigationBar.tintColor = tintColor;
    } else {
        self.navigationController.navigationBar.backItem.title = [series valueForKey:@"short_name"];
    }
    segments.tintColor = tintColor;
    ((UIToolbar *)segments.superview).tintColor = tintColor;
    currentEpisode = [episode retain];
    currentSeries = [series retain];
    self.title = [episode objectForKey:@"title"];
    overviewWebContent = [[NSString alloc] initWithFormat:[DetailViewController webOverviewFormat], [EpisodeSelectionTableController seriesColorForSeriesWithIdentifier:[series valueForKey:@"short_name"] asWeb:YES], [[currentEpisode objectForKey:@"overview_image"] base64Encoding], [currentEpisode valueForKey:@"title"], [currentSeries valueForKey:@"name"], [currentEpisode valueForKey:@"season_number"], [currentEpisode valueForKey:@"production_number"], [currentEpisode valueForKey:@"disc_number"], [currentEpisode valueForKey:@"air_date"], [currentEpisode valueForKey:@"mission_date"], [currentEpisode valueForKey:@"short_description"]];
    detailWebContent = [[NSString alloc] initWithFormat:[DetailViewController webDetailFormat], [currentEpisode objectForKey:@"description"]];
    
    //Remove unused tabs
    if(overviewSegmentIndex == -1) {
        overviewSegmentIndex = 0;
        detailSegmentIndex = 1;
        imagesSegmentIndex = 2;
        originalSegmentIndex = 3;
        BOOL showOriginalButton = [[NSUserDefaults standardUserDefaults] stringForKey:@"show_original_button"] == nil || [[NSUserDefaults standardUserDefaults] boolForKey:@"show_original_button"];
        BOOL showImagesButton = [[appDelegate dataController] intForProperty:@"show_images_button"];
        if(!showOriginalButton) {
            [segments removeSegmentAtIndex:originalSegmentIndex animated:NO];
            originalSegmentIndex = -1;
        }
        if(!showImagesButton) {
            [segments removeSegmentAtIndex:imagesSegmentIndex animated:NO];
            originalSegmentIndex--;
            imagesSegmentIndex = -1;
        }
    }
    [self clickedButton];
}

- (IBAction)clickedButton {
    if(segments.selectedSegmentIndex == overviewSegmentIndex) {
        textView.hidden = NO;
        imagesView.hidden = YES;
        [textView loadHTMLString:overviewWebContent baseURL:nil];
    } else if (segments.selectedSegmentIndex == detailSegmentIndex) {
        textView.hidden = NO;
        imagesView.hidden = YES;
        [textView loadHTMLString:detailWebContent baseURL:nil];
    } else if (segments.selectedSegmentIndex == imagesSegmentIndex) {
        [self initImagesView];
        imagesView.hidden = NO;
        textView.hidden = YES;
        [self clearTextView];
    } else if(segments.selectedSegmentIndex == originalSegmentIndex) {
        [[UIApplication sharedApplication] openURL: [NSURL URLWithString:[currentEpisode valueForKey:@"url"]]];
    }
}

- (void) initImagesView {
    if(imagesView == nil) {
        imagesView = [[EpisodeImageView alloc] initWithFrame:textView.frame andAppDelegate:appDelegate];
        [self.view addSubview:imagesView];
    }
    [imagesView displayEpisode:[self currentEpisodeId]];
}

- (void) clearTextView {
    [textView loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:@"about:blank" relativeToURL:nil]]];
}

- (void)viewDidLoad {
    //Helper variable so that removing of tabs can be done late and lazy
    overviewSegmentIndex = -1;
    textView.detectsPhoneNumbers = NO;
}

- (NSUInteger) currentEpisodeId {
    return [(NSNumber *)[currentEpisode valueForKey:@"id"] integerValue];
}

- (void) viewDidDisappear:(BOOL)animated {
    [imagesView clearEpisode];
    [self clearTextView];
    //Hack to store current episode id… before quit
    if(animated) {
        [currentEpisode release];currentEpisode=nil;
    }
    [currentSeries release];currentSeries=nil;
    [overviewWebContent release];overviewWebContent=nil;
    [detailWebContent release];detailWebContent=nil;
    segments.selectedSegmentIndex=0;
}

- (void)dealloc {
    [segments release];
    [textView release];
    [appDelegate release];
    if(imagesView != nil) {
        [imagesView release];
    }
    if(currentEpisode != nil) {
        [currentEpisode release];
    }
    [super dealloc];
}

+ (NSString *) webOverviewFormat {
    static NSString *overviewFormat;
    if(!overviewFormat) {
        overviewFormat = [[NSString alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"episode_overview" ofType:@"xml"]];
    }
    return overviewFormat;
}

+ (NSString *) webDetailFormat {
    static NSString *detailFormat;
    if(!detailFormat) {
        detailFormat = [[NSString alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"episode_detail" ofType:@"xml"]];
    }
    return detailFormat;
}

@end
